@@ -1,6 +1,6 @@
 # abstract_tool.py
 #
-# Copyright 2018-2020 Romain F. T.
+# Copyright 2018-2021 Romain F. T.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -52,7 +52,7 @@ class AbstractAbstractTool():
 		self.try_build_pane()
 
 	############################################################################
-	# Actions for tool's options ###############################################
+	# Utilities managing actions for tool's options ############################
 
 	def add_tool_action_simple(self, action_name, callback):
 		"""Convenient wrapper method adding a stateless action to the window. It
@@ -70,7 +70,7 @@ class AbstractAbstractTool():
 		return self.window.options_manager.get_value(action_name)
 
 	def set_action_sensitivity(self, action_name, state):
-		self.get_image().set_action_sensitivity(action_name, state)
+		self.window.lookup_action(action_name).set_enabled(state)
 
 	def update_actions_state(self):
 		self.set_action_sensitivity('main_color', self.use_color)
@@ -82,17 +82,29 @@ class AbstractAbstractTool():
 		return self.window._settings
 
 	############################################################################
-	# UI #######################################################################
+	# Various utilities ########################################################
 
 	def show_error(self, error_text):
 		self.window.prompt_message(True, error_text)
 
+	############################################################################
+	# Bottom pane and menubar integration ######################################
+
 	def try_build_pane(self):
 		pass
 
 	def build_bottom_pane(self):
 		return None
 
+	def on_apply_temp_pixbuf_tool_operation(self, *args):
+		pass # implemented only by transform tools
+
+	def get_options_label(self):
+		return _("No options")
+
+	def adapt_to_window_size(self, available_width):
+		pass
+
 	def add_item_to_menu(self, tools_menu):
 		tools_menu.append(self.label, 'win.active_tool::' + self.id)
 
@@ -107,8 +119,8 @@ class AbstractAbstractTool():
 	def get_edition_status(self):
 		return self.label
 
-	def get_options_label(self):
-		return _("No options")
+	############################################################################
+	# Side pane ################################################################
 
 	def build_row(self):
 		"""Build the GtkRadioButton for the sidebar. This method stores it as
@@ -117,7 +129,7 @@ class AbstractAbstractTool():
 		                        draw_indicator=False, valign=Gtk.Align.CENTER, \
 		                                                tooltip_text=self.label)
 		self.row.set_detailed_action_name('win.active_tool::' + self.id)
-		self.label_widget = Gtk.Label(label=self.label)
+		self.label_widget = Gtk.Label(label=self.label) #, use_underline=True)
 		if self.get_settings().get_boolean('big-icons'):
 			size = Gtk.IconSize.LARGE_TOOLBAR
 		else:
@@ -144,9 +156,6 @@ class AbstractAbstractTool():
 			size = Gtk.IconSize.SMALL_TOOLBAR
 		image.set_from_icon_name(self.icon_name, size)
 
-	def adapt_to_window_size(self, available_width):
-		pass
-
 	############################################################################
 	# Activation or not ########################################################
 
@@ -164,9 +173,6 @@ class AbstractAbstractTool():
 		self.non_destructive_show_modif()
 		self._ongoing_operation = False
 
-	def has_ongoing_operation(self):
-		return self._ongoing_operation
-
 	def give_back_control(self, preserve_selection):
 		self.restore_pixbuf()
 		self.non_destructive_show_modif()
@@ -174,6 +180,9 @@ class AbstractAbstractTool():
 	############################################################################
 	# History ##################################################################
 
+	def has_ongoing_operation(self):
+		return self._ongoing_operation
+
 	def do_tool_operation(self, operation):
 		pass
 
