@@ -1,6 +1,6 @@
 # image.py
 #
-# Copyright 2018-2020 Romain F. T.
+# Copyright 2018-2021 Romain F. T.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,8 +20,13 @@ from gi.repository import Gtk, Gdk, Gio, GdkPixbuf, Pango
 from .history_manager import DrHistoryManager
 from .selection_manager import DrSelectionManager
 from .properties import DrPropertiesDialog
+from .utilities import InvalidFileFormatException
+
+from .gi_composites import GtkTemplate
 
 class DrMotionBehavior():
+	_LIMIT = 10
+
 	HOVER = 0
 	DRAW = 1
 	SLIP = 2
@@ -29,23 +34,24 @@ class DrMotionBehavior():
 class NoPixbufNoChangeException(Exception):
 	def __init__(self, pb_name):
 		# Context: an error message
-		super().__init__(_("New pixbuf empty, no change applied to %s") % pb_name)
+		message = _("New pixbuf empty, no change applied to %s")
+		super().__init__(message % pb_name)
 
 ################################################################################
 
-@Gtk.Template(resource_path='/com/github/maoschanz/drawing/ui/image.ui')
+@GtkTemplate(ui='/com/github/maoschanz/drawing/ui/image.ui')
 class DrImage(Gtk.Box):
 	__gtype_name__ = 'DrImage'
 
-	_drawing_area = Gtk.Template.Child()
-	_h_scrollbar = Gtk.Template.Child()
-	_v_scrollbar = Gtk.Template.Child()
+	_drawing_area = GtkTemplate.Child()
+	_h_scrollbar = GtkTemplate.Child()
+	_v_scrollbar = GtkTemplate.Child()
 
-	CLOSING_PRECISION = 10
 	SCALE_FACTOR = 1.0 # XXX doesn't work well enough to be anything else
 
 	def __init__(self, window, **kwargs):
 		super().__init__(**kwargs)
+		self.init_template()
 		self.window = window
 
 		self.gfile = None
@@ -93,8 +99,8 @@ class DrImage(Gtk.Box):
 		rgba = self.window._settings.get_strv('ui-background-rgba')
 		self._bg_rgba = (float(rgba[0]), float(rgba[1]), \
 		                                         float(rgba[2]), float(rgba[3]))
-		# We remember this data here for performance, since it's used by the
-		# on_draw method which is called a lot.
+		# We remember this data here for performance: it's used by the `on_draw`
+		# method which is called a lot, and reading a gsettings costs a lot.
 
 	def _update_zoom_behavior(self, *args):
 		self._ctrl_to_zoom = self.window._settings.get_boolean('ctrl-zoom')
@@ -112,10 +118,10 @@ class DrImage(Gtk.Box):
 		self.scroll_y = 0
 		self.set_zoom_level(100) # will do `self.zoom_level = 1.0`
 		self.motion_behavior = DrMotionBehavior.HOVER
-		self.press2_x = 0.0
-		self.press2_y = 0.0
-		self.drag_scroll_x = 0.0
-		self.drag_scroll_y = 0.0
+		self._slip_press_x = 0.0
+		self._slip_press_y = 0.0
+		self._slip_init_x = 0.0
+		self._slip_init_y = 0.0
 
 		# Selection initialization
 		self.selection = DrSelectionManager(self)
@@ -126,18 +132,8 @@ class DrImage(Gtk.Box):
 		self.set_action_sensitivity('redo', False)
 
 	def init_background(self, width, height, background_rgba):
-		r = float(background_rgba[0])
-		g = float(background_rgba[1])
-		b = float(background_rgba[2])
-		a = float(background_rgba[3])
-		op = {
-			'tool_id': None,
-			'pixbuf': None,
-			'red': r, 'green': g, 'blue': b, 'alpha': a,
-			'width': width, 'height': height
-		}
 		self.init_image_common()
-		self._history.initial_operation = op
+		self._history.set_initial_operation(background_rgba, None, width, height)
 		self.restore_first_pixbuf()
 
 	def try_load_pixbuf(self, pixbuf):
@@ -183,17 +179,8 @@ class DrImage(Gtk.Box):
 		if not pixbuf.get_has_alpha():
 			pixbuf = pixbuf.add_alpha(False, 255, 255, 255)
 		background_rgba = self.window._settings.get_strv('default-rgba')
-		r = float(background_rgba[0])
-		g = float(background_rgba[1])
-		b = float(background_rgba[2])
-		a = float(background_rgba[3])
-		op = {
-			'tool_id': None,
-			'pixbuf': pixbuf,
-			'red': r, 'green': g, 'blue': b, 'alpha': a,
-			'width': pixbuf.get_width(), 'height': pixbuf.get_height()
-		}
-		self._history.initial_operation = op
+		self._history.set_initial_operation(background_rgba, pixbuf, \
+		                                pixbuf.get_width(), pixbuf.get_height())
 		self.set_main_pixbuf(pixbuf)
 
 	def reload_from_disk(self):
@@ -213,8 +200,19 @@ class DrImage(Gtk.Box):
 		self.remember_current_state()
 
 	def try_load_file(self, gfile):
-		self.gfile = gfile
-		pixbuf = GdkPixbuf.Pixbuf.new_from_file(self.get_file_path())
+		try:
+			self.gfile = gfile
+			pixbuf = GdkPixbuf.Pixbuf.new_from_file(self.get_file_path())
+		except Exception as ex:
+			if not ex.message:
+				ex.message = "[exception without a valid message]"
+			ex = InvalidFileFormatException(ex.message, gfile.get_path())
+			self.window.prompt_message(True, ex.message)
+			self.gfile = None
+			pixbuf = self._new_blank_pixbuf(100, 100)
+			# XXX dans l'idéal on devrait ne rien ouvrir non ? ou si besoin (si
+			# ya pas de fenêtre) ouvrir un truc respectant les settings, plutôt
+			# qu'un petit pixbuf corrompu
 		self.try_load_pixbuf(pixbuf)
 
 	############################################################################
@@ -358,7 +356,8 @@ class DrImage(Gtk.Box):
 		# Zoom level
 		cairo_context.scale(self.zoom_level, self.zoom_level)
 
-		# TODO transform tools may appreciate to draw *before* the image
+		# TODO transform tools may appreciate to draw *before* the image (issue
+		# when the canvas becomes larger)
 
 		# Image (with scroll position)
 		cairo_context.set_source_surface(self.get_surface(), \
@@ -373,14 +372,16 @@ class DrImage(Gtk.Box):
 		self._drawing_area, if the button is the mouse wheel the colors are
 		exchanged, otherwise the signal is transmitted to the selected tool."""
 		if self._is_pressed:
+			# reject attempts to draw with a given button if an other one is
+			# already doing something
 			return
 		event_x, event_y = self.get_event_coords(event)
 		if event.button == 2:
 			self.motion_behavior = DrMotionBehavior.SLIP
-			self.press2_x = event_x
-			self.press2_y = event_y
-			self.drag_scroll_x = event_x
-			self.drag_scroll_y = event_y
+			self._slip_press_x = event.x
+			self._slip_press_y = event.y
+			self._slip_init_x = self.scroll_x
+			self._slip_init_y = self.scroll_y
 			return
 		self.motion_behavior = DrMotionBehavior.DRAW
 		self.active_tool().on_press_on_area(event, self.surface, event_x, event_y)
@@ -392,28 +393,31 @@ class DrImage(Gtk.Box):
 		If a button (not the mouse wheel) is pressed, the tool's method should
 		have an effect on the image, otherwise it shouldn't change anything
 		except the mouse cursor icon for example."""
-		event_x, event_y = self.get_event_coords(event)
+
 		if self.motion_behavior == DrMotionBehavior.HOVER:
-			# TODO ça apprécierait sans doute d'avoir direct les bonnes coordonnées ?
+			# Some tools need the coords in the image, others need the coords on
+			# the widget, so the entire event is given.
 			self.active_tool().on_unclicked_motion_on_area(event, self.surface)
+
 		elif self.motion_behavior == DrMotionBehavior.DRAW:
 			# implicitely impossible if not self._is_pressed
+			event_x, event_y = self.get_event_coords(event)
 			self.active_tool().on_motion_on_area(event, self.surface, event_x, event_y)
 			self.update() # <<< comment this for better perfs
+
 		else: # self.motion_behavior == DrMotionBehavior.SLIP:
-			delta_x = int(self.drag_scroll_x - event_x)
-			delta_y = int(self.drag_scroll_y - event_y)
-			self.add_deltas(delta_x, delta_y, 0.8)
-			self.drag_scroll_x = event_x
-			self.drag_scroll_y = event_y
+			self.scroll_x = self._slip_init_x
+			self.scroll_y = self._slip_init_y
+			delta_x = self._slip_press_x - event.x
+			delta_y = self._slip_press_y - event.y
+			self.add_deltas(delta_x, delta_y, 1 / self.zoom_level)
 
 	def on_release_on_area(self, area, event):
 		"""Signal callback. Executed when a mouse button is released on
 		self._drawing_area, if the button is not the signal is transmitted to
 		the selected tool."""
 		if self.motion_behavior == DrMotionBehavior.SLIP:
-			if abs(self.press2_x - self.drag_scroll_x) < self.CLOSING_PRECISION \
-			and abs(self.press2_y - self.drag_scroll_y) < self.CLOSING_PRECISION:
+			if not self._is_slip_moving():
 				self.window.options_manager.on_middle_click()
 			self.motion_behavior = DrMotionBehavior.HOVER
 			return
@@ -423,6 +427,13 @@ class DrImage(Gtk.Box):
 		self.window.set_picture_title()
 		self._is_pressed = False
 
+	def _is_slip_moving(self):
+		"""Tells if the pointer moved while the middle button of the mouse is
+		pressed, depending on a constant hardcoded limit."""
+		mx = abs(self._slip_init_x - self.scroll_x) > DrMotionBehavior._LIMIT
+		my = abs(self._slip_init_y - self.scroll_y) > DrMotionBehavior._LIMIT
+		return mx or my
+
 	def update(self):
 		# print('image.py: _drawing_area.queue_draw')
 		# TODO immensément utilisée, mais qui ne scale pas : ça gêne clairement
@@ -679,32 +690,5 @@ class DrImage(Gtk.Box):
 		self.scroll_y = 0
 
 	############################################################################
-	# Printing #################################################################
-
-	def print_image(self):
-		op = Gtk.PrintOperation()
-		# FIXME the preview doesn't work, i guess it's because of flatpak ?
-		# I could connect to the 'preview' signal but that would be weird
-		op.connect('draw-page', self.do_draw_page)
-		op.connect('begin-print', self.do_begin_print)
-		op.connect('end-print', self.do_end_print)
-		res = op.run(Gtk.PrintOperationAction.PRINT_DIALOG, self.window)
-
-	def do_end_print(self, *args):
-		pass
-
-	def do_draw_page(self, op, print_ctx, page_num):
-		# TODO if it's too big for one page ?
-		cairo_context = print_ctx.get_cairo_context()
-		Gdk.cairo_set_source_pixbuf(cairo_context, self.main_pixbuf, 0, 0)
-		cairo_context.paint()
-
-	def do_begin_print(self, op, print_ctx):
-		op.set_n_pages(1)
-		cairo_context = print_ctx.get_cairo_context()
-		Gdk.cairo_set_source_pixbuf(cairo_context, self.main_pixbuf, 0, 0)
-		cairo_context.paint()
-
-	############################################################################
 ################################################################################
 
