@@ -1,6 +1,6 @@
 # tool_line.py
 #
-# Copyright 2018-2020 Romain F. T.
+# Copyright 2018-2021 Romain F. T.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,8 +27,8 @@ class ToolLine(AbstractClassicTool):
 		self.use_operator = True
 
 		self.add_tool_action_enum('line_shape', 'round')
-		self.add_tool_action_boolean('use_dashes', False)
-		self.add_tool_action_boolean('is_arrow', False)
+		self.add_tool_action_enum('dashes-type', 'none')
+		self.add_tool_action_enum('arrow-type', 'none')
 		self.add_tool_action_boolean('use_gradient', False)
 		self._set_options_attributes() # Not optimal but more readable
 
@@ -36,9 +36,6 @@ class ToolLine(AbstractClassicTool):
 		state_as_string = self.get_option_value('line_shape')
 		if state_as_string == 'thin':
 			self._cap_id = cairo.LineCap.BUTT
-			self._shape_label = _("Thin")
-		elif state_as_string == 'square':
-			self._cap_id = cairo.LineCap.SQUARE
 			self._shape_label = _("Square")
 		else:
 			self._cap_id = cairo.LineCap.ROUND
@@ -48,19 +45,21 @@ class ToolLine(AbstractClassicTool):
 		return _("Line options")
 
 	def _set_options_attributes(self):
-		self._use_dashes = self.get_option_value('use_dashes')
-		self._use_arrow = self.get_option_value('is_arrow')
+		self._dashes_type = self.get_option_value('dashes-type')
+		self._arrow_type = self.get_option_value('arrow-type')
 		self._use_gradient = self.get_option_value('use_gradient')
 		self._set_active_shape()
 
 	def get_edition_status(self):
 		self._set_options_attributes()
+		is_arrow = self._arrow_type != 'none'
+		use_dashes = self._dashes_type != 'none'
 		label = self.label
-		if self._use_arrow and self._use_dashes:
+		if is_arrow and use_dashes:
 			label = label + ' - ' + _("Dashed arrow")
-		elif self._use_arrow:
+		elif is_arrow:
 			label = label + ' - ' + _("Arrow")
-		elif self._use_dashes:
+		elif use_dashes:
 			label = label + ' - ' + _("Dashed")
 		return label
 
@@ -88,8 +87,8 @@ class ToolLine(AbstractClassicTool):
 			'operator': self._operator,
 			'line_width': self.tool_width,
 			'line_cap': self._cap_id,
-			'use_dashes': self._use_dashes,
-			'use_arrow': self._use_arrow,
+			'dashes_type': self._dashes_type,
+			'arrow_type': self._arrow_type,
 			'use_gradient': self._use_gradient,
 			'is_preview': is_preview,
 			'x_release': event_x,
@@ -102,7 +101,6 @@ class ToolLine(AbstractClassicTool):
 	def do_tool_operation(self, operation):
 		cairo_context = self.start_tool_operation(operation)
 		cairo_context.set_operator(operation['operator'])
-		cairo_context.set_line_cap(operation['line_cap'])
 		line_width = operation['line_width']
 		cairo_context.set_line_width(line_width)
 		c1 = operation['rgba']
@@ -118,19 +116,24 @@ class ToolLine(AbstractClassicTool):
 			cairo_context.set_source(pattern)
 		else:
 			cairo_context.set_source_rgba(c1.red, c1.green, c1.blue, c1.alpha)
-		if operation['use_dashes']:
-			cairo_context.set_dash([2 * line_width, 2 * line_width])
+
+		self.set_dashes_and_cap(cairo_context, line_width, \
+		                        operation['dashes_type'], operation['line_cap'])
+
+		if operation['arrow_type'] == 'double':
+			utilities_add_arrow_triangle(cairo_context, x1, y1, x2, y2, line_width)
+
 		# We don't memorize the path because all coords are here anyway for the
-		# linear grandient and/or the arrow.
+		# linear gradient and/or the arrow.
 		cairo_context.move_to(x1, y1)
 		cairo_context.line_to(x2, y2)
 
+		if operation['arrow_type'] != 'none':
+			utilities_add_arrow_triangle(cairo_context, x2, y2, x1, y1, line_width)
+
 		self.stroke_with_operator(operation['operator'], cairo_context, \
 		                                    line_width, operation['is_preview'])
 
-		if operation['use_arrow']:
-			utilities_add_arrow_triangle(cairo_context, x2, y2, x1, y1, line_width)
-
 	############################################################################
 ################################################################################
 
