@@ -1,6 +1,6 @@
 # tool_pencil.py
 #
-# Copyright 2018-2020 Romain F. T.
+# Copyright 2018-2021 Romain F. T.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,20 +30,16 @@ class ToolPencil(AbstractClassicTool):
 		self._shape_label = _("Round")
 		self._cap_id = cairo.LineCap.ROUND
 		self._join_id = cairo.LineCap.ROUND
-		self._use_dashes = False
+		self._dashes_type = 'none'
 
 		self.add_tool_action_enum('line_shape', 'round')
-		self.add_tool_action_boolean('use_dashes', self._use_dashes)
+		self.add_tool_action_enum('dashes-type', self._dashes_type)
 
 	def _set_active_shape(self, *args):
 		state_as_string = self.get_option_value('line_shape')
 		if state_as_string == 'thin':
 			self._cap_id = cairo.LineCap.BUTT
 			self._join_id = cairo.LineJoin.BEVEL
-			self._shape_label = _("Thin")
-		elif state_as_string == 'square':
-			self._cap_id = cairo.LineCap.SQUARE
-			self._join_id = cairo.LineJoin.MITER
 			self._shape_label = _("Square")
 		else:
 			self._cap_id = cairo.LineCap.ROUND
@@ -54,10 +50,11 @@ class ToolPencil(AbstractClassicTool):
 		return _("Pencil options")
 
 	def get_edition_status(self):
-		self._use_dashes = self.get_option_value('use_dashes')
+		self._dashes_type = self.get_option_value('dashes-type')
+		use_dashes = self._dashes_type != 'none'
 		self._set_active_shape()
 		label = self.label
-		if self._use_dashes:
+		if use_dashes:
 			label = label + ' - ' + _("Dashed")
 		return label
 
@@ -98,7 +95,7 @@ class ToolPencil(AbstractClassicTool):
 			'line_width': self.tool_width,
 			'line_cap': self._cap_id,
 			'line_join': self._join_id,
-			'use_dashes': self._use_dashes,
+			'dashes_type': self._dashes_type,
 			'is_preview': True,
 			'path': self._path
 		}
@@ -108,13 +105,12 @@ class ToolPencil(AbstractClassicTool):
 		if operation['path'] is None:
 			return
 		cairo_context = self.start_tool_operation(operation)
-		cairo_context.set_line_cap(operation['line_cap'])
-		cairo_context.set_line_join(operation['line_join'])
+		cairo_context.set_line_join(operation['line_join']) # XXX useless?
 		line_width = operation['line_width']
 		rgba = operation['rgba']
 		cairo_context.set_source_rgba(rgba.red, rgba.green, rgba.blue, rgba.alpha)
-		if operation['use_dashes']:
-			cairo_context.set_dash([2*line_width, 2*line_width])
+		self.set_dashes_and_cap(cairo_context, line_width, \
+		                        operation['dashes_type'], operation['line_cap'])
 		utilities_smooth_path(cairo_context, operation['path'])
 		self.stroke_with_operator(operation['operator'], cairo_context, \
 		                                    line_width, operation['is_preview'])
