@@ -1,6 +1,6 @@
 # window.py
 #
-# Copyright 2018-2020 Romain F. T.
+# Copyright 2018-2021 Romain F. T.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -54,6 +54,7 @@ from .deco_manager import DrDecoManagerMenubar, \
                           DrDecoManagerHeaderbar, \
                           DrDecoManagerToolbar
 from .saving_manager import DrSavingManager
+from .printing_manager import DrPrintingManager
 
 from .utilities import utilities_add_filechooser_filters
 from .utilities import utilities_gfile_is_image
@@ -122,6 +123,7 @@ class DrWindow(Gtk.ApplicationWindow):
 		self.minimap = DrMinimap(self, None)
 		self.options_manager = DrOptionsManager(self)
 		self.saving_manager = DrSavingManager(self)
+		self.printing_manager = DrPrintingManager(self)
 
 		self.add_all_win_actions()
 		self._init_tools()
@@ -300,7 +302,7 @@ class DrWindow(Gtk.ApplicationWindow):
 			# qu'un petit pixbuf rouge
 		else:
 			new_image.init_background(width, height, background_rgba)
-		self.update_tabs_visibility()
+		self._update_tabs_visibility()
 		self.notebook.set_current_page(self.notebook.get_n_pages()-1)
 
 	def on_active_tab_changed(self, *args):
@@ -347,7 +349,7 @@ class DrWindow(Gtk.ApplicationWindow):
 			if not is_saved:
 				return False
 		self.notebook.remove_page(index)
-		self.update_tabs_visibility()
+		self._update_tabs_visibility()
 		return True
 
 	def action_close_tab(self, *args):
@@ -399,6 +401,10 @@ class DrWindow(Gtk.ApplicationWindow):
 		self.notebook.connect('switch-page', self.on_active_tab_changed)
 
 		# Managing drag-and-drop
+		if self.app.runs_in_sandbox:
+			return # no dnd with actual file paths in the flatpak sandbox
+			# XXX i could test if the app has home:ro permissions instead? eg to
+			# help during development (where builder does have the permission)
 		self.notebook.drag_dest_set(Gtk.DestDefaults.ALL, [], Gdk.DragAction.MOVE)
 		self.notebook.connect('drag-data-received', self.on_data_dropped)
 		self.notebook.drag_dest_add_uri_targets()
@@ -698,7 +704,7 @@ class DrWindow(Gtk.ApplicationWindow):
 		self.info_action.set_action_name(action_name)
 		self.info_action.set_label(action_label)
 
-	def update_tabs_visibility(self):
+	def _update_tabs_visibility(self):
 		should_show = (self.notebook.get_n_pages() > 1) and not self.fullscreened
 		self.notebook.set_show_tabs(should_show)
 
@@ -721,7 +727,7 @@ class DrWindow(Gtk.ApplicationWindow):
 		self.tools_flowbox.set_visible(not state)
 		self.toolbar_box.set_visible(not state) # XXX not if empty!!
 		self.fullscreen_btn.set_visible(state)
-		self.update_tabs_visibility()
+		self._update_tabs_visibility()
 
 	############################################################################
 	# SIDE PANE (TOOLS) ########################################################
@@ -812,8 +818,8 @@ class DrWindow(Gtk.ApplicationWindow):
 			self._build_options_menu()
 			self._adapt_to_window_size()
 		except Exception as e:
-			self.prompt_message(True, _("Error: pane invalid for '%s', " + \
-			                   "please report this bug.") % self.active_tool_id)
+			self.prompt_message(True, _("Error loading the bottom pane for " + \
+			    "the tool '%s', please report this bug.") % self.active_tool_id)
 			print(e)
 
 	def _update_fullscreen_icon(self):
@@ -830,6 +836,9 @@ class DrWindow(Gtk.ApplicationWindow):
 		return self.tools[self.former_tool_id]
 
 	def back_to_previous(self, *args):
+		if self.former_tool_id == self.active_tool_id:
+			self.force_selection()
+			# avoid cases where applying a transform tool keeps the tool active
 		self.tools[self.former_tool_id].row.set_active(True)
 
 	def _build_options_menu(self):
@@ -955,11 +964,11 @@ class DrWindow(Gtk.ApplicationWindow):
 		for uri in uris:
 			try:
 				gfile = Gio.File.new_for_uri(uri)
-				is_image, error_msg = utilities_gfile_is_image(gfile)
+				is_valid_image, error_msg = utilities_gfile_is_image(gfile)
 			except Exception as excp:
-				is_image = False
+				is_valid_image = False
 				error_msg = excp.message
-			if is_image:
+			if is_valid_image:
 				gfiles.append(gfile)
 			else:
 				self.prompt_message(True, error_msg)
@@ -1008,7 +1017,8 @@ class DrWindow(Gtk.ApplicationWindow):
 		return self.saving_manager.save_current_image(True, True, False, True)
 
 	def action_print(self, *args):
-		self.get_active_image().print_image()
+		pixbuf = self.get_active_image().main_pixbuf
+		self.printing_manager.print_pixbuf(pixbuf)
 
 	def action_export_cb(self, *args):
 		cb = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
